// PopupWnd.cpp: CPopupWnd NX̃Cve[V
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "PopupWnd.h"
#include "MomoUtil.h"

#define WINDOWCLASS					"PopupWnd"

#define POPUPWNDM_REFRESHTEXT		WM_APP + 1

#define TIMERID_CLOSE				0
#define TIMERID_FADEIN				1
#define TIMERID_FADEOUT				2

// init@C
#define KEY_TITLE					"Title"
#define KEY_VIEWFILENAME			"ViewFileName"
#define KEY_VIEWMODE				"ViewMode"
#define KEY_LEFT					"Left"
#define KEY_RIGHT					"Right"
#define KEY_TOP						"Top"
#define KEY_BOTTOM					"Bottom"
#define KEY_VIEWTIME				"ViewTime"
#define KEY_CHANGENOTIFY			"ChangeNotify"
#define KEY_FONTNAME				"FontName"
#define KEY_FONTSIZE				"FontSize"
#define KEY_TEXTCOLOR				"TextColor"
#define KEY_BKCOLOR					"BkColor"
#define KEY_ANIMATIONMODE			"AnimationMode"
#define KEY_ANIMATIONSPEED			"AnimationSpeed"

#define DEF_VIEWFILENAME			"output.txt"
#define DEF_VIEWMODE				VIEWMODE_BROWSER
#define DEF_WIDTH					300
#define DEF_HEIGHT					300
#define DEF_VIEWTIME				10
#define DEF_CHANGENOTIFY			1
#define DEF_FONTNAME				"lr SVbN"
#define DEF_FONTSIZE				9
#define DEF_TEXTCOLOR				0x00000000
#define DEF_BKCOLOR					0x00ffffff
#define DEF_ANIMATIONMODE			ANIMATIONMODE_FADEIN
#define DEF_ANIMATIONSPEED			40

LRESULT CALLBACK WindowProc(HWND, UINT, WPARAM, LPARAM);
LRESULT CALLBACK SubWindowProc(HWND, UINT, WPARAM, LPARAM);

ATOM PopupWndRegisterClass(HINSTANCE hInstance)
{
	WNDCLASSEX wcex;

	wcex.cbSize = sizeof(WNDCLASSEX); 

	wcex.style			= CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc	= (WNDPROC)WindowProc;
	wcex.cbClsExtra		= 0;
	wcex.cbWndExtra		= 0;
	wcex.hInstance		= hInstance;
	wcex.hIcon			= NULL;
	wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground	= (HBRUSH)(COLOR_WINDOW+1);
	wcex.lpszMenuName	= "IDR_MENU1";
	wcex.lpszClassName	= WINDOWCLASS;
	wcex.hIconSm		= NULL;

	return RegisterClassEx( &wcex );
}


//////////////////////////////////////////////////////////////////////
// \z/
//////////////////////////////////////////////////////////////////////

CPopupWnd::CPopupWnd(HINSTANCE hInstance)
{
	Init(hInstance, "Default");
}

CPopupWnd::CPopupWnd(HINSTANCE hInstance, LPCSTR lpSectionName)
{
	Init(hInstance, lpSectionName);
}

CPopupWnd::~CPopupWnd()
{
}

//////////////////////////////////////////////////////////////////////
// \bh
//////////////////////////////////////////////////////////////////////
BOOL CPopupWnd::Show(int nCmdShow, BOOL fAnimation)
{
	LoadSetting(m_SectionName);

	if (m_hWnd) {
		Refresh();

	} else {
		m_hWnd = CreateWindowEx(WS_EX_TOOLWINDOW | WS_EX_TOPMOST
							  , WINDOWCLASS, m_Title
							  , WS_BORDER | WS_SYSMENU | WS_THICKFRAME
							  , m_Rect.left, m_Rect.top
							  , m_Rect.right - m_Rect.left, m_Rect.bottom - m_Rect.top
							  , NULL, NULL, m_hInstance, this);

		if(!m_hWnd) {
			return FALSE;
		}

		if (fAnimation) {
			switch (m_AnimationMode) {
				case ANIMATIONMODE_FADEIN: {
					int h = GetSystemMetrics(SM_CYMINTRACK);
					SetWindowPos(m_hWnd, 0, m_Rect.left, m_Rect.bottom - h
										  , m_Rect.right - m_Rect.left, h
										  , SWP_NOZORDER | SWP_NOACTIVATE);
					SetTimer(m_hWnd, TIMERID_FADEIN
							, (m_Rect.bottom - m_Rect.top - h) / m_AnimationSpeed, NULL);
					break;
				}
			}
		}
		ShowWindow(m_hWnd, nCmdShow);
		UpdateWindow(m_hWnd);
	}

	return TRUE;
}

void CPopupWnd::Refresh(void)
{
	OnRefreshText(m_hWnd);
}

void CPopupWnd::Close(BOOL fAnimation)
{
	KillTimer(m_hWnd, TIMERID_CLOSE);
	SaveSetting();

	if (fAnimation) {
		switch (m_AnimationMode) {
			case ANIMATIONMODE_FADEIN: {
				int h = GetSystemMetrics(SM_CYMINTRACK);
				SetTimer(m_hWnd, TIMERID_FADEOUT
						, (m_Rect.bottom - m_Rect.top - h) / m_AnimationSpeed, NULL);
				return;
			}
		}
	}
	SendMessage(m_hWnd, WM_CLOSE, 0, 0);
}

void CPopupWnd::StartCloseTimer(void)
{
	if (m_ViewTime > 0) {
		SetTimer(m_hWnd, TIMERID_CLOSE, 500, NULL);
		m_CloseTime = GetTickCount() + (m_ViewTime * 1000);
	}
}

LPCSTR CPopupWnd::GetTitle(LPSTR lpBuff)
{
	lstrcpy(lpBuff, m_Title);
	return lpBuff;
}

HWND CPopupWnd::GethWnd(void)
{
	return m_hWnd;
}

BOOL CPopupWnd::CheckChanged(void)
{
	if (!m_ChangeNotify) return FALSE;

	FILETIME FileTime;
	MomoUtil::GetFileTime(m_ViewFileName, &FileTime);
	if (CompareFileTime(&FileTime, &m_ViewFileTime) > 0) {
		CopyMemory(&m_ViewFileTime, &FileTime, sizeof(FILETIME));
		return TRUE;
	}
	return FALSE;
}


//////////////////////////////////////////////////////////////////////
// bZ[W
//////////////////////////////////////////////////////////////////////
LRESULT CPopupWnd::WndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch (msg) {
		HANDLE_MSG(hWnd, WM_CREATE, OnCreate);
		HANDLE_MSG(hWnd, WM_ACTIVATE, OnActivate);
		HANDLE_MSG(hWnd, WM_ACTIVATEAPP, OnActivateApp);
		HANDLE_MSG(hWnd, WM_SIZE, OnSize);
		HANDLE_MSG(hWnd, WM_CTLCOLORSTATIC, OnCtlColor);
		HANDLE_MSG(hWnd, WM_KEYDOWN, OnKey);
		HANDLE_MSG(hWnd, WM_COMMAND, OnCommand);
		HANDLE_MSG(hWnd, WM_TIMER, OnTimer);
		HANDLE_MSG(hWnd, WM_DESTROY, OnDestroy);

	//ƎbZ[W̏
	case POPUPWNDM_REFRESHTEXT:
		OnRefreshText(hWnd);
		break;

	default:
		return DefWindowProc(hWnd, msg, wParam, lParam);
	}
	return 0;
}

LRESULT CPopupWnd::SubWndProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch (msg) {
		HANDLE_MSG(hWnd, WM_KEYDOWN, OnKey);

	default:
		return CallWindowProc(m_hEditPrevWndProc, hWnd, msg, wParam, lParam);
	}
	return 0;
}

BOOL CPopupWnd::OnCreate(HWND hWnd, LPCREATESTRUCT lpCreateStruct)
{
	RECT rc;
	GetClientRect(hWnd, &rc);

	switch (m_ViewMode) {
		case VIEWMODE_EDIT: {
			//EDITEBhE
			m_hEdit = CreateWindow("Edit", "File Not Found."
								, WS_CHILD | WS_VISIBLE | WS_VSCROLL
								| ES_MULTILINE | ES_WANTRETURN | ES_READONLY
								, 0, 0, rc.right, rc.bottom
								, hWnd, (HMENU)0, m_hInstance, NULL);
			if (!m_hEdit) {
				return FALSE;
			}

			//tHgݒ
			HDC hDC = GetDC(m_hEdit);
			int nHeight = -MulDiv(m_FontSize, GetDeviceCaps(hDC, LOGPIXELSY), 72);
			HFONT m_hFont = CreateFont(nHeight, 0, 0, 0
										, FW_REGULAR, FALSE, FALSE, FALSE
										, SHIFTJIS_CHARSET
										, OUT_DEFAULT_PRECIS
										, CLIP_DEFAULT_PRECIS
										, PROOF_QUALITY
										, FIXED_PITCH | FF_MODERN
										, m_FontName);
			if (m_hFont) {
				SetWindowFont(m_hEdit, m_hFont, TRUE);
			}
			ReleaseDC(m_hEdit, hDC);

			//wiFpuV쐬
			m_hBkBrush = CreateSolidBrush(m_BkColor);
		} break;

		case VIEWMODE_BROWSER: {
			m_hEdit = CreateWindow("AtlAxWin", "Shell.Explorer.2"
								, WS_CHILD|WS_VISIBLE
								, 0, 0, rc.right, rc.bottom
								, hWnd, (HMENU)0, m_hInstance, NULL);
			if (!m_hEdit) {
				return FALSE;
			}
			CComPtr<IUnknown> ie;
			if (AtlAxGetControl(m_hEdit, &ie) == S_OK) {
				m_IE = ie;
			}
			if (!m_IE) {
				return FALSE;
			}
		} break;
	}

	//TuNX
	m_hEditPrevWndProc = SubclassWindow(m_hEdit, SubWindowProc);

	OnRefreshText(hWnd);

	return TRUE;
}

void CPopupWnd::OnActivateApp(HWND hWnd, BOOL fActivate, DWORD dwThreadId)
{
	if (fActivate) {
		KillTimer(hWnd, TIMERID_CLOSE);
	} else {
		StartCloseTimer();
	}
}

void CPopupWnd::OnActivate(HWND hWnd, UINT state, HWND hWndActDeact, BOOL fMinimized)
{
	switch (state) {
		case WA_ACTIVE:
			KillTimer(hWnd, TIMERID_CLOSE);
			break;	

		case WA_INACTIVE:
			StartCloseTimer();
			break;
	}
}

void CPopupWnd::OnSize(HWND hWnd, UINT state, int cx, int cy)
{
	SetWindowPos(m_hEdit, 0, 0, 0, cx, cy, SWP_NOMOVE | SWP_NOZORDER);
}

HBRUSH CPopupWnd::OnCtlColor(HWND hWnd, HDC hDC, HWND hWndChild, int type)
{
	if (type == CTLCOLOR_STATIC) {
		::SetTextColor(hDC, m_TextColor);
		::SetBkColor(hDC, m_BkColor);
		return m_hBkBrush;
	}
	return NULL;
}

void CPopupWnd::OnKey(HWND hWnd, UINT vk, BOOL fDown, int cRepeat, UINT flags)
{
	if (vk == VK_ESCAPE) {
		Close(FALSE);
	}
}

void CPopupWnd::OnCommand(HWND hWnd, int id, HWND hwndCtl, UINT codeNotify)
{
}


void CPopupWnd::OnTimer(HWND hWnd, UINT id)
{
	switch (id) {
		case TIMERID_CLOSE: {
			DWORD now = GetTickCount();
			if (now > m_CloseTime) {
				Close(TRUE);
			}
		}
		break;
		case TIMERID_FADEIN: {
			RECT rc;
			int h = (m_Rect.bottom - m_Rect.top - GetSystemMetrics(SM_CYMINTRACK)) / m_AnimationSpeed;
			GetWindowRect(m_hWnd, &rc);
			SetWindowPos(m_hWnd, 0
						, rc.left, rc.top - h
						, rc.right - rc.left, (rc.bottom - rc.top) + h
						, SWP_NOZORDER | SWP_NOACTIVATE);
			GetWindowRect(m_hWnd, &rc);
			if (rc.top < m_Rect.top) {
				SetWindowPos(m_hWnd, 0
							, m_Rect.left, m_Rect.top
							, m_Rect.right - m_Rect.left, m_Rect.bottom - m_Rect.top
							, SWP_NOZORDER | SWP_NOACTIVATE);
				KillTimer(m_hWnd, TIMERID_FADEIN);
			}
		}
		break;
		case TIMERID_FADEOUT: {
			RECT rc;
			int h = (m_Rect.bottom - m_Rect.top - GetSystemMetrics(SM_CYMINTRACK)) / m_AnimationSpeed;
			GetWindowRect(m_hWnd, &rc);
			SetWindowPos(m_hWnd, 0
						, rc.left, rc.top + h
						, rc.right - rc.left, (rc.bottom - rc.top) - h
						, SWP_NOZORDER | SWP_NOACTIVATE);
			GetWindowRect(m_hWnd, &rc);
			if ((rc.bottom - rc.top) <= GetSystemMetrics(SM_CYMINTRACK)) {
				KillTimer(m_hWnd, TIMERID_FADEOUT);
				SendMessage(m_hWnd, WM_CLOSE, 0, 0);
			}
		}
		break;
	}
}

void CPopupWnd::OnDestroy(HWND hWnd)
{
	SaveSetting();

	if (m_hEditPrevWndProc) {
		SubclassWindow(m_hEdit, m_hEditPrevWndProc);
		m_hEditPrevWndProc = NULL;
	}

	KillTimer(hWnd, TIMERID_CLOSE);
	m_IE.Release();

	DeleteObject(m_hFont);
	DeleteObject(m_hBkBrush);

	m_hWnd = NULL;
}


void CPopupWnd::OnRefreshText(HWND hWnd)
{
	switch (m_ViewMode) {
		case VIEWMODE_EDIT: {
			//t@C̓e\
			HANDLE hFile = CreateFile(m_ViewFileName
							 , GENERIC_READ, 0
							 , 0, OPEN_EXISTING
							 , 0, NULL);
			if (hFile == INVALID_HANDLE_VALUE) {
				SetWindowText(m_hEdit, "t@CJ܂łB");
				break;
			}
			ULONG uLen = GetFileSize(hFile, NULL);
			LPSTR lpBuff = new char[uLen+1];
			ReadFile(hFile, lpBuff, uLen, &uLen, NULL);
			lpBuff[uLen] = '\0';
			SetWindowText(m_hEdit, lpBuff);
			CloseHandle(hFile);
			delete[] lpBuff;
			break;
		}

		case VIEWMODE_BROWSER: {
			CComVariant vempty, vUrl(m_ViewFileName);
			m_IE->Navigate2(&vUrl, &vempty, &vempty, &vempty, &vempty);
			break;
		}
	}
}

//////////////////////////////////////////////////////////////////////
// ֐
//////////////////////////////////////////////////////////////////////
void CPopupWnd::Init(HINSTANCE hInstance, LPCSTR lpSectionName)
{
	m_hInstance = hInstance;
	m_hWnd = NULL;
	m_hEdit = NULL;
	m_hEditPrevWndProc = NULL;
	LoadSetting(lpSectionName);
}

void CPopupWnd::LoadSetting(LPCSTR lpSectionName)
{
	int len = lstrlen(lpSectionName);
	MoveMemory(m_SectionName, lpSectionName, MAX_SECTION - 1);
	m_SectionName[MAX_SECTION - 1] = '\0';

	char IniFileName[MAX_PATH];
	MomoUtil::GetIniFileName(m_hInstance, IniFileName);

	GetPrivateProfileString(m_SectionName, KEY_TITLE, m_SectionName, m_Title, MAX_SECTION - 1, IniFileName);

	GetPrivateProfileString(m_SectionName, KEY_VIEWFILENAME, DEF_VIEWFILENAME, m_ViewFileName, MAX_PATH - 1, IniFileName);
	MomoUtil::GetFileTime(m_ViewFileName, &m_ViewFileTime);

	m_ViewMode = GetPrivateProfileInt(m_SectionName, KEY_VIEWMODE, DEF_VIEWMODE, IniFileName);

	RECT rc;
	::SystemParametersInfo(SPI_GETWORKAREA, 0, &rc, 0);
	m_Rect.left = GetPrivateProfileInt(m_SectionName, KEY_LEFT, rc.right - DEF_WIDTH, IniFileName);
	m_Rect.right = GetPrivateProfileInt(m_SectionName, KEY_RIGHT, rc.right, IniFileName);
	m_Rect.top = GetPrivateProfileInt(m_SectionName, KEY_TOP, rc.bottom - DEF_HEIGHT, IniFileName);
	m_Rect.bottom = GetPrivateProfileInt(m_SectionName, KEY_BOTTOM, rc.bottom, IniFileName);

	m_ViewTime = GetPrivateProfileInt(m_SectionName, KEY_VIEWTIME, DEF_VIEWTIME, IniFileName);

	m_ChangeNotify = (BOOL)GetPrivateProfileInt(m_SectionName, KEY_CHANGENOTIFY, DEF_CHANGENOTIFY, IniFileName);

	GetPrivateProfileString(m_SectionName, KEY_FONTNAME, DEF_FONTNAME, m_FontName, MAX_FONTNAME - 1, IniFileName);
	m_FontSize = GetPrivateProfileInt(m_SectionName, KEY_FONTSIZE, DEF_FONTSIZE, IniFileName);
	m_TextColor = GetPrivateProfileInt(m_SectionName, KEY_TEXTCOLOR, DEF_TEXTCOLOR, IniFileName);
	m_BkColor = GetPrivateProfileInt(m_SectionName, KEY_BKCOLOR, DEF_BKCOLOR, IniFileName);

	m_AnimationMode = GetPrivateProfileInt(m_SectionName, KEY_ANIMATIONMODE, DEF_ANIMATIONMODE, IniFileName);
	m_AnimationSpeed = GetPrivateProfileInt(m_SectionName, KEY_ANIMATIONSPEED, DEF_ANIMATIONSPEED, IniFileName);

	m_fNeedSave = TRUE;
}

void CPopupWnd::SaveSetting(void)
{
	char buff[256];

	if (!m_fNeedSave) return;

	char IniFileName[MAX_PATH];
	MomoUtil::GetIniFileName(m_hInstance, IniFileName);

	WritePrivateProfileString(m_SectionName, KEY_TITLE, m_Title, IniFileName);
	WritePrivateProfileString(m_SectionName, KEY_VIEWFILENAME, m_ViewFileName, IniFileName);
	WritePrivateProfileString(m_SectionName, KEY_VIEWMODE, itoa(m_ViewMode, buff, 10), IniFileName);

	RECT rc;
	GetWindowRect(m_hWnd, &rc);
	WritePrivateProfileString(m_SectionName, KEY_LEFT, itoa(rc.left, buff, 10), IniFileName);
	WritePrivateProfileString(m_SectionName, KEY_RIGHT, itoa(rc.right, buff, 10), IniFileName);
	WritePrivateProfileString(m_SectionName, KEY_TOP, itoa(rc.top, buff, 10), IniFileName);
	WritePrivateProfileString(m_SectionName, KEY_BOTTOM, itoa(rc.bottom, buff, 10), IniFileName);

	WritePrivateProfileString(m_SectionName, KEY_VIEWTIME, itoa(m_ViewTime, buff, 10), IniFileName);
	WritePrivateProfileString(m_SectionName, KEY_CHANGENOTIFY, itoa(m_ChangeNotify, buff, 10), IniFileName);

	WritePrivateProfileString(m_SectionName, KEY_FONTNAME, m_FontName, IniFileName);
	WritePrivateProfileString(m_SectionName, KEY_FONTSIZE, itoa(m_FontSize, buff, 10), IniFileName);
	wsprintf(buff, "0x%08x", m_TextColor);
	WritePrivateProfileString(m_SectionName, KEY_TEXTCOLOR, buff, IniFileName);
	wsprintf(buff, "0x%08x", m_BkColor);
	WritePrivateProfileString(m_SectionName, KEY_BKCOLOR, buff, IniFileName);

	WritePrivateProfileString(m_SectionName, KEY_ANIMATIONMODE, itoa(m_AnimationMode, buff, 10), IniFileName);
	WritePrivateProfileString(m_SectionName, KEY_ANIMATIONSPEED, itoa(m_AnimationSpeed, buff, 10), IniFileName);
	m_fNeedSave = FALSE;
}

//////////////////////////////////////////////////////////////////////
// EBhEvV[W
//////////////////////////////////////////////////////////////////////
LRESULT CALLBACK WindowProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	//popupNX̎擾
	CPopupWnd *popup;
	if (msg == WM_CREATE) {
		LPCREATESTRUCT cs = (LPCREATESTRUCT)lParam;
		popup = (CPopupWnd *)cs->lpCreateParams;
		SetWindowLong(hWnd, GWL_USERDATA, (LONG)popup);
	} else {
		popup = (CPopupWnd *)GetWindowLong(hWnd, GWL_USERDATA);
	}

	//NX̃EBhEvV[W
	return popup->WndProc(hWnd, msg, wParam, lParam);
}

LRESULT CALLBACK SubWindowProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	//popupNX̎擾
	CPopupWnd *popup = (CPopupWnd *)GetWindowLong(GetParent(hWnd), GWL_USERDATA);

	//NX̃EBhEvV[W
	return popup->SubWndProc(hWnd, msg, wParam, lParam);
}

